"""
Extract forecasts from intelliO3 and store them for MLAir

"""

__author__ = "Felix Kleinert"
__date__ = "2021-01-29"

import os
import xarray as xr
import shutil

from mlair.configuration.path_config import check_path_and_create
from mlair.reference_models.abstract_reference_model import AbstractReferenceB2share


class IntelliO3_ts_v1(AbstractReferenceB2share):
    """
    Reference handler that extracts IntelliO3-ts v1.0 forecasts (Kleinert, 2021).

    IntelliO3 forecasts can be used as a competitive model within MLAir. Downloads the IntelliO3 tar-ball and extracts
    the forecasts.

    Kleinert, F., Leufen, L. H., and Schultz, M. G.: IntelliO3-ts v1.0: a neural network approach to predict
    near-surface ozone concentrations in Germany, Geosci. Model Dev., 14, 1–25,
    https://doi.org/10.5194/gmd-14-1-2021, 2021.
    """

    def __init__(self, ref_name: str, ref_store_path: str = None):
        """
        :param ref_name: Desired Name of reference forecast
        :type ref_name: str
        :param ref_store_path: Path to store reference forecasts
        :type ref_store_path: str
        """
        super().__init__(b2share_hosturl="https://b2share.eudat.eu",
                         b2share_bucket="0cae9db2-f388-4136-8d28-9d9c5665d641",
                         b2share_key="IntelliO3-ts.tar.gz",
                         )
        self.ref_name = ref_name
        if ref_store_path is None:
            ref_store_path = f"{self.ref_name}/"
        self.ref_store_path = ref_store_path
        self.tmp_extract_path = "tmp_downloads/"
        self.orig_forecast_path = "IntelliO3-ts/IntelliO3-ts_network/forecasts/"
        self.file_pattern = "forecasts_DE?????_test.nc"

    def untar_forecasts(self):
        """
        Extracts IntelliO3 forecasts from tar-ball.
        """
        cmd = f"tar -xf {self.tmp_extract_path}{self.b2share_key} --directory {self.tmp_extract_path} --wildcards --no-anchored '{self.orig_forecast_path}{self.file_pattern}'"
        os.system(cmd)

    def file_list(self):
        """
        :return: base dir of tmp path and list of forecast files
        :rtype: tuple(str, list(str))
        """
        for base_dir, dirs, file_names in os.walk(self.tmp_extract_path + self.orig_forecast_path):
            pass
        return base_dir, file_names

    def read_and_drop(self, sel_coords: dict = None):
        """
        Reads original forecast files, renames coord type and store forecasts as NetCdf4 files
        :param sel_coords:
        """
        if sel_coords is None:
            sel_coords = {'type': ['CNN']}
        in_path, files = self.file_list()
        check_path_and_create(self.ref_store_path)
        for infile in files:
            data = xr.open_dataarray(os.path.join(in_path, infile))
            data = data.sel(**sel_coords)
            data.coords['type'] = ['nn']
            data.to_netcdf(os.path.join(self.ref_store_path, infile))

    def make_reference_available_locally(self, remove_tmp_dir: bool = True):
        """

        :return:
        :rtype:
        """
        if not self.is_reference_available_locally(self.ref_store_path):
            if not os.path.exists(self.tmp_extract_path+self.b2share_key):
                self.download_from_b2share(tmp_download_path=self.tmp_extract_path)
            self.untar_forecasts()
            self.read_and_drop()
            if remove_tmp_dir:
                shutil.rmtree(self.tmp_extract_path)


if __name__ == '__main__':
    io3 = IntelliO3_ts_v1('IntelliO3-ts')
    io3.make_reference_available_locally()

